@@ -487,4 +487,74 @@ rawToBase64 <- function(x) {
   base64encode(x)
 }
 
-.globals <- new.env()
\ No newline at end of file
+.globals <- new.env()
+
+
+#' Create an HTTP/WebSocket daemonized server (experimental)
+#' 
+#' Creates an HTTP/WebSocket server on the specified host and port. The server is daemonized
+#' so R interactive sessions are not blocked to handle requests.
+#' 
+#' @param host A string that is a valid IPv4 address that is owned by this 
+#'   server, or \code{"0.0.0.0"} to listen on all IP addresses.
+#' @param port A number or integer that indicates the server port that should be
+#'   listened on. Note that on most Unix-like systems including Linux and Mac OS
+#'   X, port numbers smaller than 1025 require root privileges.
+#' @param app A collection of functions that define your application. See 
+#'   Details.
+#' @return A handle for this server that can be passed to
+#'   \code{\link{stopDaemonizedServer}} to shut the server down.
+#'   
+#' @details In contrast to servers created by \code{\link{startServer}}, calls to \code{\link{service}} 
+#'   are not needed to accept and handle connections. If the port
+#'   cannot be bound (most likely due to permissions or because it is already
+#'   bound), an error is raised.
+#'   
+#'   The \code{app} parameter is where your application logic will be provided 
+#'   to the server. This can be a list, environment, or reference class that 
+#'   contains the following named functions/methods:
+#'   
+#'   \describe{
+#'     \item{\code{call(req)}}{Process the given HTTP request, and return an 
+#'     HTTP response. This method should be implemented in accordance with the
+#'     \href{https://github.com/jeffreyhorner/Rook/blob/a5e45f751/README.md}{Rook}
+#'     specification.}
+#'     \item{\code{onHeaders(req)}}{Optional. Similar to \code{call}, but occurs
+#'     when headers are received. Return \code{NULL} to continue normal
+#'     processing of the request, or a Rook response to send that response,
+#'     stop processing the request, and ask the client to close the connection.
+#'     (This can be used to implement upload size limits, for example.)}
+#'     \item{\code{onWSOpen(ws)}}{Called back when a WebSocket connection is established.
+#'     The given object can be used to be notified when a message is received from
+#'     the client, to send messages to the client, etc. See \code{\link{WebSocket}}.}
+#'   }
+#'   
+#'   The \code{startPipeServer} variant is not supported yet.
+#' @seealso \code{\link{startServer}}
+#' @export
+startDaemonizedServer <- function(host, port, app) {
+  server <- startServer(host, port, app)
+  tryCatch({
+    server <- daemonize(server)
+  }, error=function(e) {
+    stopServer(server)
+    stop(e)
+  })
+  return(server)
+}
+
+#' Stop a running daemonized server in Unix environments
+#' 
+#' Given a handle that was returned from a previous invocation of 
+#' \code{\link{startDaemonizedServer}}, closes all open connections for that server,
+#' removes listeners in the R event loop and 
+#' unbinds the port. \strong{Be careful not to call \code{stopDaemonizedServer} more than 
+#' once on a handle, as this will cause the R process to crash!}
+#' 
+#' @param server A handle that was previously returned from
+#'   \code{\link{startDaemonizedServer}}.
+#'   
+#' @export
+stopDaemonizedServer <- function(server) {
+  destroyDaemonizedServer(server)
+}
